﻿-- Action Bar Profiles
PAB = LibStub("AceAddon-3.0"):NewAddon("ProfilesActionBars", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

function PAB:OnInitialize()
   PABdb = LibStub("AceDB-3.0"):New("ProfilesActionBars",nil,"Default")
end


function PAB:Echo(text)
if PROFILES_TYST then return end
   DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFProfiles: Action bars - "..text)
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function PAB:PreSaveABProfile(name)
	PAB:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function PAB:SaveABSProfile(name)

	-- Save a restore point if empty db
	if not PAB:ExistsProfile("Restore [LOCKED]") and not PABSAVERESTORE then 
		PABSAVERESTORE=1 
		PAB:SaveABSProfile("Restore [LOCKED]") 
		PABSAVERESTORE=nil
	end
	
	-- User trying to overwrite restore point!
	if name == "Restore [LOCKED]" and not PABSAVERESTORE then 
		PAB:Echo("This profile is locked at the first run and can only be loaded!")
		return nil
	end
	
	PABdb:SetProfile(name)

	-- Solving /mp save tank X 2
	if PAB:ExistsProfile(name) then PABdb:ResetProfile(name) end
	
   PAB:Echo("saving!")
   
	for actionID=1, 120 do
      
		local type_ab, id, subType, extraID = GetActionInfo(actionID)

		if type_ab and id then
			
			
			if type_ab == "flyout" then	-- Save name if flyout
				local flyOutName = GetFlyoutInfo(id)
				PABdb.profile[actionID] = {type_ab, flyOutName, actionID}
			
			else						-- Save spellID
				PABdb.profile[actionID] = {type_ab, id, actionID}
			end
		end
   end
   
   return name
end

-- This is an alias for ProfilesHub
function PAB:Save(...)
	PAB:SaveABSProfile(...)
end

-- ProfilesHub: Send message to all slaves that we are now switching profile
function PAB:PreLoadABProfile(name)

	
	-- No such profile saved
	if not PAB:ExistsProfile(name) then PAB:Echo(name.." profile not found.") return end

	-- In combat!
	if InCombatLockdown() then PAB:Echo("Cannot change macros in combat!") return end

	PAB:SendAceMsg("load",name) -- ProfilesHub
end

-- This function should not be called directly.
-- Should only be called by slave event!

function PAB:LoadABSProfile(name)

	-- No such profile saved
	if not PAB:ExistsProfile(name) then PAB:Echo(name.." profile not found.") return nil end
	
	-- In combat!
	if InCombatLockdown() then PAB:Echo("Cannot change macros in combat!") return end

	-- Save a restore point if empty db
	if not PAB:ExistsProfile("Restore [LOCKED]") and not PABSAVERESTORE then 
		PABSAVERESTORE=1 
		PAB:SaveABSProfile("Restore [LOCKED]")
		PABSAVERESTORE=nil
	end
	
	
	PABdb:SetProfile(name)

	PAB:Echo("Loading action from profile "..name.."!")
	
	-- Start fresh with nothing on the cursor
	ClearCursor()
	
	
	--Load Action bar profile!
	for index, keys in pairs(PABdb.profile) do
		
		if index ~= "numMacros" then
			
			-- Make sure we dont have anything on the cursor
			ClearCursor()

			local type_ab, action_id, subType, extraID = GetActionInfo(index)
			
			if index == 1 then
					print(type_ab,keys[1],action_id,keys[2])
				end
			
			-- Clear current spot if not duplicate with saved
			if type_ab ~= keys[1] and action_id ~= keys[2] then
			
				
				PickupAction(index)
				
				-- Do not touch macros!
				local cursorInfoType = GetCursorInfo()
				if cursorInfoType == "macro" then PlaceAction(index) end
				
				-- Clear current spot if not duplicate with saved
				if index == 1 then
					print("->"..cursorInfoType,keys[1],keys[2])
				end
			
				ClearCursor()
				
				
				if keys[1] == "spell" 		 then PickupSpell(keys[2]) end
				
				if keys[1] == "equipmentset" then PickupEquipmentSetByName(keys[2]) end
				
				if keys[1] == "item"         then 
				
					-- Do we have this item equiped ?
					if IsEquippedItem(keys[2]) then 
						PickupItem(keys[2])
					else
						-- Do we have item in inventory ?
						for bagNumber = -1, 11 do 
							--allow checking of bank slots, otherwise 0 to 4
							local nSlots = GetContainerNumSlots(bagNumber);
							for slotNumber = 1, nSlots do
								local id = GetContainerItemID(bagNumber, slotNumber);
								if id == keys[2] then 
									PickupItem(keys[2])
									break
								end
							end
						end
					end
					
				end
				
				if keys[1] == "companion"	 then
					local found_critter = nil
					-- Go through companions and see if we can find our saved SpellID
					for i=1,GetNumCompanions("CRITTER") do
						_,_,companionID = GetCompanionInfo("CRITTER", i)
						
						if keys[2] == companionID then
							PickupCompanion("CRITTER",i)
							found_critter = 1
							break
						end
					end
					
					if not found_critter then
						for i=1,GetNumCompanions("MOUNT") do
							_,_,companionID = GetCompanionInfo("MOUNT", i)
							
							if keys[2] == companionID then
								PickupCompanion("MOUNT",i)
								break
							end
						end
					end
				end
				
				if keys[1] == "flyout" then
					PickupSpellBookItem(PAB:GetSpellID_byName(keys[2]), BOOKTYPE_SPELL)
				end
				
				PlaceAction(index)
			end
		end
	end
	
	PAB:SendMessage("ABS_Profiles_load",name) -- ProfilesHub
	return name
end

-- This is an alias for ProfilesHub
function PAB:Load(...)
	PAB:LoadABSProfile(...)
end

-- Cache spells from book
-- Used mainly for flyout spells
PAB_spellCache = {}
function PAB:GetSpellID_byName(spellName)
   
   if PAB_spellCache[spellName] then
      return PAB_spellCache[spellName]
   else
      -- Build cache
      for book=1, MAX_SKILLLINE_TABS do
         local _, _, offset, numSpells = GetSpellTabInfo(book)
         
         for i=1, numSpells do
            local index = offset + i
            local spell, stance = GetSpellBookItemName(index, BOOKTYPE_SPELL)
            
            -- This way we restore the max rank of spells
            PAB_spellCache[spell] = index
            
            if( stance and stance ~= "" ) then
               PAB_spellCache[spell .. stance] = index
            end
         end
      end
      
      return PAB_spellCache[spellName]
   end
end

function PAB:ExistsProfile(name)
   for i, k in ipairs(PABdb:GetProfiles()) do
      local key = k
      
      if key == name then
         return (key)
      end
   end
   return nil
end

function PAB:ListProfiles()
    
	for i,k in ipairs(PABdb:GetProfiles()) do
		if k == PABdb:GetCurrentProfile() then
			PAB:Echo(' - ' .. k.." (current)")
		else
			PAB:Echo(' - ' .. k)
		end
	end
end

-- ProfilesHub
function PAB:PreDeleteProfile(name)
	PAB:SendAceMsg("delete",name) -- ProfilesHub
end

function PAB:DeleteProfil(name)

	-- User trying to overwrite restore point!
	if name == "Restore [LOCKED]" and not PABSAVERESTORE then 
		PAB:Echo("This profile cannot be deleted! It is locked at the first run and can only be loaded!")
		return nil
	end
	
	if (PAB:ExistsProfile(name)==nil)then
		PAB:Echo(name.." doesnt exist.")
		PAB:ListProfiles()
		return nil
	else
		if name == PABdb:GetCurrentProfile() then
			PABdb:SetProfile('Default')
		end
		PABdb:DeleteProfile(name)
		PAB:Echo(name.." deleted.")
		
		return name
	end
end

-- This is an alias for ProfilesHub
function PAB:Delete(...)
	PAB:DeleteProfil(...)
end

SLASH_PAB1 = "/pab"
SlashCmdList["PAB"] = function(msg)
   local cmd, arg = string.split(" ", msg, 2)
   cmd = string.lower(cmd or "")
   arg = arg or ""
   
   if (cmd == "version") then
      print("Keyboard Layout Saver v."..KLSversion)
   elseif (cmd=="list")then
      for i,k in ipairs(PABdb:GetProfiles()) do
		if k == PABdb:GetCurrentProfile() then
			print(' - ' .. k.." (current)")
		else
			print(' - ' .. k)
		end
	end
   elseif (cmd=="" or arg=="")then
      PAB:Echo("/PAB delete <name>")
      PAB:Echo("/PAB save <name>")
      PAB:Echo("/PAB load <name>")
      PAB:Echo("/PAB list")
      PAB:Echo("/PAB version")
      PAB:Echo("/PAB debug [on|off]")
   elseif (cmd=="save" and arg~="")then
      PAB:PreSaveABProfile(arg)
   elseif (cmd=="load" and arg~="")then
      PAB:PreLoadABProfile(arg)
   elseif (cmd=="delete" and arg~="")then
      PAB:PreDeleteProfile(arg)
   
   else    
      PAB:Echo("wrong entry")
   end
end